<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class LastErrorTime
 *
 * Created via:
 * oitc migrations create -p DistributeModule SatelliteTasks
 *
 * Run migration:
 * oitc migrations migrate -p DistributeModule
 *
 */
class SatelliteTasks extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {

        $this->table('satellite_tasks')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('satellite_id', 'integer', [
                'default' => null,
                'null'    => false
            ])
            ->addColumn('task', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => false,
            ])
            ->addColumn('status', 'integer', [
                'null'    => false,
                'default' => 0
            ])
            ->addColumn('result', 'text', [
                'default' => null,
                'limit'   => \Phinx\Db\Adapter\MysqlAdapter::TEXT_LONG,
                'null'    => true,
            ])
            ->addColumn('error', 'text', [
                'default' => null,
                'limit'   => \Phinx\Db\Adapter\MysqlAdapter::TEXT_LONG,
                'null'    => true,
                'after'   => 'result'
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->addIndex(['satellite_id'], ['unique' => false])
            ->create();

    }
}
