<?php
declare(strict_types=1);

namespace DistributeModule\Command;

use App\Model\Table\ContainersTable;
use App\Model\Table\HostsTable;
use Cake\Command\Command;
use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\TableRegistry;
use DistributeModule\Model\Table\SatellitesTable;

/**
 * CheckContainerId command.
 */
class CheckContainerIdCommand extends Command {
    /**
     * Hook method for defining this command's option parser.
     *
     * @see https://book.cakephp.org/4/en/console-commands/commands.html#defining-arguments-and-options
     * @param \Cake\Console\ConsoleOptionParser $parser The parser to be defined
     * @return \Cake\Console\ConsoleOptionParser The built parser.
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        $parser = parent::buildOptionParser($parser);

        return $parser;
    }

    /**
     * Implement this method with your command's logic.
     *
     * @param \Cake\Console\Arguments $args The command arguments.
     * @param \Cake\Console\ConsoleIo $io The console io
     * @return null|void|int The exit code or null for success
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        /** @var $ContainersTable ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var $SatellitesTable SatellitesTable */
        $SatellitesTable = TableRegistry::getTableLocator()->get('DistributeModule.Satellites');

        $hosts = $HostsTable->find()
            ->select([
                'id',
                'uuid',
                'name',
                'container_id',
                'satellite_id'
            ])
            ->where([
                'satellite_id >' => 0
            ])
            ->disableHydration()
            ->all();

        $satellites = $SatellitesTable->find()
            ->select([
                'id',
                'name',
                'container_id'
            ])
            ->disableHydration()
            ->all();

        $satellites = $satellites->toArray();

        if (empty($satellites)) {
            //No Satellites - nothing to check
            return;
        }

        $satellitesToContainer = [];
        foreach ($satellites as $satellite) {
            $satellitesToContainer[$satellite['id']] = (int)$satellite['container_id'];
        }


        $resolvedContainerCache = [];
        $errorHosts = [];
        foreach ($hosts as $host) {
            $containerId = $host['container_id'];
            if (!isset($resolvedContainerCache[$containerId])) {
                $resolvedContainerCache[$containerId] = $ContainersTable->resolveChildrenOfContainerIds($containerId);
            }

            $containerIds = $resolvedContainerCache[$containerId];

            $satelliteContainerId = $satellitesToContainer[$host['satellite_id']];

            if (!in_array($satelliteContainerId, $containerIds, true)) {
                $errorHosts[] = $host;
            }
        }

        if(empty($errorHosts)){
            $io->success('Nothing to do - you are already prepared for openITCOCKPIT 4.4!');
        }else{
            $io->hr();
            $io->error('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
            $io->error('! IMPORTANT BREAKING CHANGES ARE COMING !');
            $io->error('! With openITCOCKPIT 4.4 Satellites are !');
            $io->error('! are bound to the Hosts container_id   !');
            $io->error('! The folowing list of hosts will have  !');
            $io->error('! a broken configuration. Please move   !');
            $io->error('! the satellite into a higher level     !');
            $io->error('! container.                            !');
            $io->error('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
            $io->hr();
            $io->error('List of broken/affected hosts');
            foreach($errorHosts as $errorHost){
                $io->error(sprintf(' - %s [%s]', $errorHost['name'], $errorHost['uuid']));
            }

            $io->out('To see this list again, run the command: "oitc DistributeModule.check_container_id"');
            $io->out('Continue in 10 seconds...');
            sleep(10);
        }


    }
}
