<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace DistributeModule\Model\Entity;

use Cake\I18n\FrozenTime;
use Cake\ORM\Entity;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

/**
 * Satellite Entity
 *
 * @property int $id
 * @property string $name
 * @property string $address
 * @property int $container_id
 * @property int $sync_instance
 * @property string|null $timezone
 * @property string|null $login
 * @property int $port
 * @property string|null $private_key_path
 * @property int $remote_port
 * @property string|null $url
 * @property int $use_proxy
 * @property string|null $proxy_url
 * @property int $verify_certificate
 * @property string|null $api_key
 * @property int|null $interval
 * @property int|null $timeout
 * @property int $use_timesync
 * @property \Cake\I18n\FrozenTime $created
 * @property \Cake\I18n\FrozenTime $modified
 * @property string $sync_method
 * @property bool $nsta_sync_instance
 *
 * @property \App\Model\Entity\Container $container
 * @property \App\Model\Entity\Host[] $hosts
 * @property Map[] $maps
 * @property SatelliteStatus $satellite_status
 */
class Satellite extends Entity {
    /**
     * Fields that can be mass assigned using newEntity() or patchEntity().
     *
     * Note that when '*' is set to true, this allows all unspecified fields to
     * be mass assigned. For security purposes, it is advised to set '*' to false
     * (or remove it), and explicitly make individual fields accessible as needed.
     *
     * @var array
     */
    protected $_accessible = [
        'name'               => true,
        'address'            => true,
        'container_id'       => true,
        'sync_instance'      => true,
        'timezone'           => true,
        /* SSH */
        'login'              => true,
        'port'               => true,
        'private_key_path'   => true,
        'remote_port'        => true,
        /* HTTPS */
        'url'                => true,
        'use_proxy'          => true,
        'proxy_url'          => true,
        'verify_certificate' => true,
        'api_key'            => true,
        'interval'           => true,
        'timeout'            => true,
        'use_timesync'       => true,
        'nsta_sync_instance' => true,

        'created'          => true,
        'modified'         => true,
        'sync_method'      => true,
        'maps'             => true,
        'satellite_status' => true
    ];

    public function toAngularArray(?UserTime $UserTime = null) {
        $record = $this->toArray();

        $syncStatusHuman = [
            SatelliteStatus::SatelliteStatusNil    => __('Unknown'),
            SatelliteStatus::SatelliteStatusOK     => __('Success'),
            SatelliteStatus::SatelliteStatusFailed => __('Failed')
        ];

        if (!isset($record['satellite_status'])) {
            $record['satellite_status'] = [
                'status'      => SatelliteStatus::SatelliteStatusNil,
                'last_export' => null,
                'last_seen'   => null
            ];
        } else {
            if ($UserTime !== null) {
                $satelliteStatus = $this->get('satellite_status');
                $lastExport = $satelliteStatus->get('last_export');
                $lastSeen = $satelliteStatus->get('last_seen');

                if ($lastExport instanceof FrozenTime) {
                    $record['satellite_status']['last_export'] = $UserTime->format($lastExport->getTimestamp());
                }

                if ($lastSeen instanceof FrozenTime) {
                    $record['satellite_status']['last_seen'] = $UserTime->format($lastSeen->getTimestamp());
                }
            }
        }

        $record['satellite_status']['status_human'] = $syncStatusHuman[$record['satellite_status']['status']];

        return $record;
    }
}
