<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace DistributeModule\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * SatelliteStatus Model
 *
 * @property \DistributeModule\Model\Table\SatellitesTable&\Cake\ORM\Association\BelongsTo $Satellites
 *
 * @method \DistributeModule\Model\Entity\SatelliteStatus newEmptyEntity()
 * @method \DistributeModule\Model\Entity\SatelliteStatus newEntity(array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[] newEntities(array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus get($primaryKey, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \DistributeModule\Model\Entity\SatelliteStatus[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class SatelliteStatusTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('satellite_status');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->belongsTo('Satellites', [
            'foreignKey' => 'satellite_id',
            'joinType'   => 'INNER',
            'className'  => 'DistributeModule.Satellites',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        // Readonly table
        // Data gets written bei the Go version of the NSTA

        $validator
            ->integer('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->integer('status')
            ->allowEmptyString('status');

        $validator
            ->scalar('last_error')
            ->requirePresence('last_error', 'create')
            ->notEmptyString('last_error');

        $validator
            ->dateTime('last_export')
            ->requirePresence('last_export', 'create')
            ->notEmptyDateTime('last_export');

        $validator
            ->dateTime('last_seen')
            ->requirePresence('last_seen', 'create')
            ->notEmptyDateTime('last_seen');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn(['satellite_id'], 'Satellites'));

        return $rules;
    }
}
